import { Injectable } from '@nestjs/common';
import { IndexDataSchedulingManager } from './managers/index-data-scheduling.manager';
import { DataSchedulingReadService } from '../../data/services/data-scheduling-read.service';
import {
  DataSchedulingEntity,
  DataSchedulingLogEntity,
} from '../entities/data-scheduling.entity';
import { PaginationResponse } from 'src/core/response/domain/ok-response.interface';
import { BaseReadOrchestrator } from 'src/core/modules/domain/usecase/orchestrators/base-read.orchestrator';
import { DetailDataSchedulingManager } from './managers/detail-data-scheduling.manager';
import { TABLE_NAME } from 'src/core/strings/constants/table.constants';
import { IndexDataSchedulingLogManager } from './managers/index-data-scheduling-log.manager';
import { DataSchedulingLogReadService } from '../../data/services/data-scheduling-log-read.service';

@Injectable()
export class DataSchedulingReadOrchestrator extends BaseReadOrchestrator<DataSchedulingEntity> {
  constructor(
    private indexManager: IndexDataSchedulingManager,
    private indexLogManager: IndexDataSchedulingLogManager,
    private detailManager: DetailDataSchedulingManager,
    private serviceData: DataSchedulingReadService,
    private logServiceData: DataSchedulingLogReadService,
  ) {
    super();
  }

  async index(params): Promise<PaginationResponse<DataSchedulingEntity>> {
    this.indexManager.setFilterParam(params);
    this.indexManager.setService(this.serviceData, TABLE_NAME.DATA_SCHEDULING);
    await this.indexManager.execute();
    return this.indexManager.getResult();
  }

  async indexLog(params): Promise<PaginationResponse<DataSchedulingLogEntity>> {
    this.indexLogManager.setFilterParam(params);
    this.indexLogManager.setService(
      this.logServiceData,
      TABLE_NAME.DATA_SCHEDULING_LOG,
    );
    await this.indexLogManager.execute();
    return this.indexLogManager.getResult();
  }

  async detail(dataId: string): Promise<DataSchedulingEntity> {
    this.detailManager.setData(dataId);
    this.detailManager.setService(this.serviceData, TABLE_NAME.DATA_SCHEDULING);
    await this.detailManager.execute();
    return this.detailManager.getResult();
  }
}
