import { EventsHandler, IEventHandler } from '@nestjs/cqrs';
import { DataSchedulingCreatedEvent } from '../../entities/event/data-scheduling-created.event';
import {
  DataSchedulingLogEntity,
  SCHEDULING_LOG_ACTION_ENUM,
  SCHEDULING_LOG_TYPE_ENUM,
} from '../../entities/data-scheduling.entity';
import { decryptionTotal } from '../../../infrastructure/helpers';
import { Logger } from '@nestjs/common';
import { DataSchedulingLogDataService } from '../../../data/services/data-scheduling-log-data.service';

@EventsHandler(DataSchedulingCreatedEvent)
export class DataSchedulingCreatedHandler
  implements IEventHandler<DataSchedulingCreatedEvent>
{
  private readonly logger = new Logger(DataSchedulingCreatedHandler.name);

  constructor(private service: DataSchedulingLogDataService) {}

  async handle(event: DataSchedulingCreatedEvent) {
    const data = event?.data?.data;
    const totalPercentage = decryptionTotal(data?.indexing_key);

    const scheduleName = data?.name || 'a new schedule';
    const description = `<div><b>${data.creator_name}</b> created <b><i>${scheduleName}</i></b> schedule from <b><i>${data?.schedule_date_from}</i></b> to <b><i>${data.schedule_date_to}</i></b> with a total percentage of <b><i>${totalPercentage}%</i></b>.</div>`;

    const payload: DataSchedulingLogEntity = {
      type: SCHEDULING_LOG_TYPE_ENUM.DATA_SCHEDULING,
      action: SCHEDULING_LOG_ACTION_ENUM.CREATE,
      log_created_at: new Date().getTime(),

      data_id: data?.id,
      name: data?.name,
      indexing_key: data?.indexing_key,
      schedule_date_from: data?.schedule_date_from,
      schedule_date_to: data?.schedule_date_to,

      status: data?.status,
      creator_id: data?.creator_id,
      creator_name: data?.creator_name,
      editor_id: data?.editor_id,
      editor_name: data?.editor_name,
      created_at: data?.created_at,
      updated_at: data?.updated_at,
      description: description,
    };

    await this.service.create(payload as any);
    this.logger.verbose(
      `[SCHEDULING LOG] Create data for ID: ${payload.data_id}`,
    );
  }
}
