import { EventsHandler, IEventHandler } from '@nestjs/cqrs';
import { DataSchedulingDeletedEvent } from '../../entities/event/data-scheduling-deleted.event';
import {
  DataSchedulingLogEntity,
  SCHEDULING_LOG_ACTION_ENUM,
  SCHEDULING_LOG_TYPE_ENUM,
} from '../../entities/data-scheduling.entity';
import { Logger } from '@nestjs/common';
import { DataSchedulingLogDataService } from '../../../data/services/data-scheduling-log-data.service';
import { decryptionTotal } from '../../../infrastructure/helpers';

@EventsHandler(DataSchedulingDeletedEvent)
export class DataSchedulingDeletedHandler
  implements IEventHandler<DataSchedulingDeletedEvent>
{
  private readonly logger = new Logger(DataSchedulingDeletedHandler.name);

  constructor(private service: DataSchedulingLogDataService) {}

  async handle(event: DataSchedulingDeletedEvent) {
    const deletedData = event?.data?.data;
    const user = event?.data?.user;

    const deleterName =
      user?.name ||
      deletedData?.editor_name ||
      deletedData?.creator_name ||
      'System';

    const scheduleName = deletedData?.name || 'an item';
    const totalPercentage = decryptionTotal(deletedData?.indexing_key);
    const description = `<div><b>${deleterName}</b> deleted schedule: <b><i>${scheduleName} (${totalPercentage}%)</i></b> schedule from <b><i>${deletedData?.schedule_date_from}</i></b> to <b><i>${deletedData.schedule_date_to}</i></b>.</div>`;

    const payload: DataSchedulingLogEntity = {
      type: SCHEDULING_LOG_TYPE_ENUM.DATA_SCHEDULING,
      action: SCHEDULING_LOG_ACTION_ENUM.DELETE,
      log_created_at: new Date().getTime(),

      data_id: deletedData?.id,
      name: deletedData?.name,
      indexing_key: deletedData?.indexing_key,
      schedule_date_from: deletedData?.schedule_date_from,
      schedule_date_to: deletedData?.schedule_date_to,
      status: deletedData?.status,

      creator_id: deletedData?.creator_id,
      creator_name: deletedData?.creator_name,
      editor_id: deletedData?.editor_id,
      editor_name: deletedData?.editor_name,
      created_at: deletedData?.created_at,
      updated_at: deletedData?.updated_at,
      description: description,
    };

    await this.service.create(payload as any);
    this.logger.verbose(
      `[SCHEDULING LOG] Delete data for ID: ${payload.data_id}`,
    );
  }
}
