import { EventsHandler, IEventHandler } from '@nestjs/cqrs';
import { ChangeDocEvent } from 'src/modules/configuration/couch/domain/events/change-doc.event';
import { TransactionSettingDataService } from '../../../data/services/sales-price-formula-data.service';
import { EncryptionHelper } from '../../helpers/encryption.helper';

@EventsHandler(ChangeDocEvent)
export class TransactionSettingHandler
  implements IEventHandler<ChangeDocEvent>
{
  constructor(
    private transactionSettingService: TransactionSettingDataService, // private orchestrator: SalesPriceFormulaDataOrchestrator,
  ) {}

  async handle(event: ChangeDocEvent) {
    const data = event.data.data;
    const database = event.data.database;

    if (database !== 'api_configuration') return;
    const value = data.value;

    const currentTransactionSetting =
      await this.transactionSettingService.getTransactionSetting();

    if (value === currentTransactionSetting.value) return;
    const key = EncryptionHelper.encrypt(`${value}`);
    const payload = {
      id: currentTransactionSetting.id,
      value: 0,
      key,
    };

    await this.transactionSettingService.updateTransactionSetting(payload);
  }
}
